from os.path import dirname, join, exists
import tensorflow as tf
import subprocess


if tf.test.is_built_with_cuda():
    shared_object = join(dirname(__file__), '../../bazel-bin/pyronn_layers/pyronn_layers.so')
    if not exists(shared_object):
        print('Trying to build bazel-bin/pyronn_layers/pyronn_layers.so')
        subprocess.run(['bazel', 'build', '//pyronn_layers:pyronn_layers.so'])


    _pyronn_layers_module = tf.load_op_library(shared_object)

        
    print('Loading shared object...')
    for obj in dir(_pyronn_layers_module):
        globals()[obj] = getattr(_pyronn_layers_module, obj)
        print('\t' + str(obj))
else:
    print('Not built with CUDA')



# -*- coding: utf-8 -*-
import glob
import os.path

import pystencils_autodiff.tensorflow_jit
import tensorflow as tf
from pkg_resources import DistributionNotFound, get_distribution

import pyronn_layers

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = 'pyronn-layers'
    __version__ = get_distribution(dist_name).version
except DistributionNotFound:
    __version__ = 'unknown'
finally:
    del get_distribution, DistributionNotFound


_pyronn_layers_dir = os.path.join(os.path.dirname(__file__), 'PYRO-NN-Layers')
_pyronn_layers_sources = glob.glob(os.path.join(_pyronn_layers_dir, '*.cc'))

assert _pyronn_layers_sources, "Could not find the source files of PYRO-NN-Layers.\n" \
    "Did you check out the Git submodule (`git submodule update --init --recursive`)?"


_pyronn_layers_module_file = pystencils_autodiff.tensorflow_jit.compile_sources_and_load(
    [],
    _pyronn_layers_sources,
    additional_compile_flags=['-I' + _pyronn_layers_dir, '-DGOOGLE_CUDA'],
    compile_only=True)  # TODO: msvc! \D
_pyronn_layers_module = tf.load_op_library(_pyronn_layers_module_file)

for obj in dir(_pyronn_layers_module):
    setattr(pyronn_layers, obj, getattr(_pyronn_layers_module, obj))



import re

###############################
#
#   Name of the folder with the contribution sources
#
###############################
package_name = 'pyronn_layers'

###############################
#
#   Build all .cc, .cu, .h files from the contribution folder (package_name) into one .so file
#
###############################
def build():
    file_path = '../../tensorflow/tools/pip_package/BUILD'
    tf_build = open(file_path,'r')
    build = tf_build.read()
    tf_build.close()
    pattern = re.compile('sh_binary.*?COMMON_PIP_DEPS.*?\],\n', re.DOTALL)

    for m in pattern.finditer(build):
        s = m.start()
        e = m.end()
        g = m.group()

        start_pos = e - 4
        build = build[:start_pos] + '"//'+package_name+':'+package_name+'.so",\n' + build[start_pos:]
        tf_build_w = open(file_path, 'w')
        tf_build_w.write(build)
        tf_build_w.flush()
        tf_build_w.close()

###################################
#
#   Recursivly copy the compiled files into the respective subfolder in preparation to build the pip_package
#
###################################

def build_pip_package():
    file_path = '../../tensorflow/tools/pip_package/build_pip_package.sh'
    tf_build = open(file_path,'r')
    build = tf_build.read()
    tf_build.close()
    pattern = re.compile('function prepare_src\(\).*?\}\n\nfunction build_wheel\(\)', re.DOTALL)

    for m in pattern.finditer(build):
        pointer = m.start()
        offset = m.group().rfind('}')


        insert_position = pointer+offset-1

        build = build[:insert_position] + '\n  cp -r '+package_name+' ${TMPDIR}\n  cp -r bazel-bin/'+package_name+'/*.so ${TMPDIR}/'+package_name + build[insert_position:]

        tf_build_w = open(file_path, 'w')
        tf_build_w.write(build)
        tf_build_w.flush()
        tf_build_w.close()

###################################
#
#   Append our .so file to the build process of the pip package
#
###################################
def setup():
    file_path = '../../tensorflow/tools/pip_package/setup.py'
    tf_build = open(file_path, 'r')
    build = tf_build.read()
    tf_build.close()
    pattern = re.compile('headers.*?setup\(\n', re.DOTALL)

    for m in pattern.finditer(build):
        pointer = m.start()
        offset = m.group().rfind(')')

        insert_position = pointer + offset +1

        build = build[:insert_position] + '\n\npackages_to_install = find_packages()\npackages_to_install.append(\''+package_name+'\')' + build[insert_position:]
        tf_build_w = open(file_path, 'w')
        tf_build_w.write(build)
        tf_build_w.flush()
        tf_build_w.close()


###################################
#
#   Tensorflow 1.9 containts a Bug for user locations than us_us, which leads to a wrong parsing of the us decimal sign '.' to ','
#    ,which later in the run process leads to wrong interpretations as next argument instead of decimal sign. This method applies
#   a valid bugfix from the tensorflow github issue:
#       https://github.com/tensorflow/tensorflow/pull/22044/commits/ce9e5b035b32ef02cd7d10f6ffdd27cc2a75664d
#   Original issue:
#       https://github.com/tensorflow/tensorflow/issues/21164
#
###################################
def location_bugfix():
    file_path = "../tensorflow/python/framework/python_op_gen_internal.cc"
    tf_build = open(file_path, 'r')
    build = tf_build.read()
    tf_build.close()
    build_updated = build.replace('return strings::StrCat(value.f());',
                                  'std::ostringstream s;\n s.imbue(std::locale::classic());\n s << value.f();\n return s.str();')
    tf_build_w = open(file_path, 'w')
    tf_build_w.write(build_updated)
    tf_build_w.flush()
    tf_build_w.close()

print('Prepare BUILD in tensorflow/tools/pip_package/')
build()
print('Prepare build_pip_package.sh in tensorflow/tools/pip_package/')
build_pip_package()
print('Prepare setup.py in tensorflow/tools/pip_package/')
setup()
print('Apply bugfix for Tensorflow regarding String parsing w.r.t to location')
location_bugfix()
print('Tensorflow is patched')



# -*- coding: utf-8 -*-
#
# Copyright © 2019 Stephan Seitz <stephan.seitz@fau.de>
#
# Distributed under terms of the GPLv3 license.

"""

"""
from shutil import copyfile


def test_import():
    import pyronn_layers


def test_compile_pyronn_layers_so():
    import pyronn_layers
    copyfile(pyronn_layers._pyronn_layers_module_file, 'pyronn_layers.so')
